﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Data;
using System.Windows.Interactivity;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.GridView;

namespace HierarchyIsExpandable
{
    public class IsExpandableBehavior : Behavior<RadGridView>
    {
        private IValueConverter converter = new IntegerToBooleanConverter();

        protected override void OnAttached()
        {
            base.OnAttached();

            this.AssociatedObject.RowLoaded += OnRowLoaded;
            this.AssociatedObject.RowUnloaded += OnRowUnloaded;
        }

        protected override void OnDetaching()
        {
            base.OnDetaching();

            this.AssociatedObject.RowLoaded -= OnRowLoaded;
            this.AssociatedObject.RowUnloaded -= OnRowUnloaded;
        }

        public string IsExpandableSourcePropertyName
        {
            get
            {
                return (string)GetValue(IsExpandableSourcePropertyNameProperty);
            }
            set
            {
                SetValue(IsExpandableSourcePropertyNameProperty, value);
            }
        }

        public static readonly DependencyProperty IsExpandableSourcePropertyNameProperty =
            DependencyProperty.Register("IsExpandableSourcePropertyName", typeof(string), typeof(IsExpandableBehavior), new PropertyMetadata(null));

        private void OnRowLoaded(object sender, RowLoadedEventArgs e)
        {
            var row = e.Row as GridViewRow;

            if (row != null)
            {
                row.SetBinding(GridViewRow.IsExpandableProperty, new Binding(this.IsExpandableSourcePropertyName) { Converter = converter });
            }
        }

        private void OnRowUnloaded(object sender, RowUnloadedEventArgs e)
        {
            var row = e.Row as GridViewRow;

            if (row != null)
            {
                BindingOperations.ClearBinding(row, GridViewRow.IsExpandableProperty);
            }
        }
    }
}
